require_relative 'bait'
require_relative 'image_database'
require_relative 'position'
require_relative 'snake'
require_relative 'constants'

module SnakeGame
  # Class for managing and updating baits (snake food)
  class BaitManager
    include Constants
    attr_accessor :baits
    def initialize
      @baits = []
      @images = ImageDatabase.new
    end

    def update(snake)
      if @baits.count.zero?
        position = random_free_position snake
        add_bait position
      else
        update_baits snake
      end
    end

    def add_bait(position)
      @baits << (Bait.new position, @images.bait)
    end

    def random_free_position(snake)
      loop do
        position = Position.new (rand * WIDTH).round(-1) % WIDTH,
                                (rand * HEIGHT).round(-1) % HEIGHT
        return position if free? position, snake
      end
    end

    def update_baits(snake)
      @baits.each do |bait|
        next if rand < EVASION
        loop do
          new_position = bait.position + calculate_random_movement
          if free? new_position, snake
            bait.position = new_position
            break
          end
        end
      end
    end

    def free?(position, snake)
      free = true
      snake.snake_parts.each do |part|
        free = false if part.position == position
      end
      @baits.each do |bait|
        free = false if bait.position == position
      end
      free
    end

    def calculate_random_movement
      random = rand
      return Position.new(STEP, 0) if random > 0.75 # go right
      return Position.new(-STEP, 0) if random > 0.5 # go left
      return Position.new(0, STEP) if random > 0.25 # go down
      Position.new(0, -STEP) # go up
    end

    def draw
      @baits.each(&:draw)
    end
  end
end
